@@ -15,6 +15,10 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with BOINC.  If not, see <http://www.gnu.org/licenses/>.
 
+#ifdef __APPLE__
+#include <Carbon/Carbon.h>
+#endif
+
 #ifdef _WIN32
 #include "boinc_win.h"
 #else
@@ -26,6 +30,7 @@
 #include <ctime>
 #include <cstdarg>
 #include <cstring>
+#include <cmath>
 #if HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
 #endif
@@ -47,15 +52,18 @@
 #include "run_app_windows.h"
 #endif
 
+#include "app_config.h"
 #include "async_file.h"
 #include "client_msgs.h"
 #include "cs_notice.h"
+#include "cs_proxy.h"
 #include "cs_trickle.h"
 #include "file_names.h"
 #include "hostinfo.h"
-#include "hostinfo_network.h"
 #include "http_curl.h"
 #include "network.h"
+#include "project.h"
+#include "result.h"
 #include "sandbox.h"
 #include "shmem.h"
 
@@ -66,6 +74,13 @@ using std::max;
 CLIENT_STATE gstate;
 COPROCS coprocs;
 
+#ifndef SIM
+#ifdef NEW_CPU_THROTTLE
+THREAD_LOCK client_mutex;
+THREAD throttle_thread;
+#endif
+#endif
+
 CLIENT_STATE::CLIENT_STATE()
     : lookup_website_op(&gui_http),
     get_current_version_op(&gui_http),
@@ -79,12 +94,14 @@ CLIENT_STATE::CLIENT_STATE()
     scheduler_op = new SCHEDULER_OP(http_ops);
 #endif
     client_state_dirty = false;
+    clock_change = false;
     check_all_logins = false;
     cmdline_gui_rpc_port = 0;
     run_cpu_benchmarks = false;
     file_xfer_giveup_period = PERS_GIVEUP;
     had_or_requested_work = false;
     tasks_suspended = false;
+    tasks_throttled = false;
     network_suspended = false;
     suspend_reason = 0;
     network_suspend_reason = 0;
@@ -96,9 +113,12 @@ CLIENT_STATE::CLIENT_STATE()
 #else
     core_client_version.prerelease = false;
 #endif
+    strcpy(language, "");
+    strcpy(client_brand, "");
     exit_after_app_start_secs = 0;
     app_started = 0;
     exit_before_upload = false;
+    run_test_app = false;
     show_projects = false;
     strcpy(detach_project_url, "");
     strcpy(main_host_venue, "");
@@ -126,11 +146,14 @@ CLIENT_STATE::CLIENT_STATE()
     redirect_io = false;
     disable_graphics = false;
     cant_write_state_file = false;
+    benchmarks_running = false;
+    device_status_time = 0;
 
-    debt_interval_start = 0;
+    rec_interval_start = 0;
     retry_shmem_time = 0;
     must_schedule_cpus = true;
     no_gui_rpc = false;
+    gui_rpc_unix_domain = false;
     new_version_check_time = 0;
     all_projects_list_check_time = 0;
     detach_console = false;
@@ -140,11 +163,19 @@ CLIENT_STATE::CLIENT_STATE()
     launched_by_manager = false;
     initialized = false;
     last_wakeup_time = dtime();
+    device_status_time = 0;
+#ifdef _WIN32
+    have_sysmon_msg = false;
+#endif
 }
 
 void CLIENT_STATE::show_host_info() {
     char buf[256], buf2[256];
 
+    msg_printf(NULL, MSG_INFO,
+        "Host name: %s",
+        host_info.domain_name
+    );
     nbytes_to_string(host_info.m_cache, 0, buf, sizeof(buf));
     msg_printf(NULL, MSG_INFO,
         "Processor: %d %s %s",
@@ -153,20 +184,37 @@ void CLIENT_STATE::show_host_info() {
     if (ncpus != host_info.p_ncpus) {
         msg_printf(NULL, MSG_INFO, "Using %d CPUs", ncpus);
     }
+#if 0
     if (host_info.m_cache > 0) {
         msg_printf(NULL, MSG_INFO,
             "Processor: %s cache",
             buf
         );
     }
+#endif
     msg_printf(NULL, MSG_INFO,
         "Processor features: %s", host_info.p_features
     );
 #ifdef __APPLE__
+    SInt32 temp;
     int major, minor, rev;
-    sscanf(host_info.os_version, "%d.%d.%d", &major, &minor, &rev);
+    OSStatus err = noErr;
+    
+    err = Gestalt(gestaltSystemVersionMajor, &temp);
+    major = temp;
+    if (!err) {
+    err = Gestalt(gestaltSystemVersionMinor, &temp);
+    minor = temp;
+    }
+    if (!err) {
+    err = Gestalt(gestaltSystemVersionBugFix, &temp);
+    rev = temp;
+    }
+    if (err) {
+        sscanf(host_info.os_version, "%d.%d.%d", &major, &minor, &rev);
+    }
     msg_printf(NULL, MSG_INFO,
-        "OS: Mac OS X 10.%d.%d (%s %s)", major-4, minor,
+        "OS: Mac OS X %d.%d.%d (%s %s)", major, minor, rev, 
         host_info.os_name, host_info.os_version
     );
 #else
@@ -195,11 +243,23 @@ void CLIENT_STATE::show_host_info() {
             "VirtualBox version: %s",
             host_info.virtualbox_version
         );
+    } else {
+#if defined (_WIN32) && !defined(_WIN64)
+        if (!strcmp(get_primary_platform(), "windows_x86_64")) {
+            msg_printf(NULL, MSG_INFO,
+                "VirtualBox: can't detect because this is a 32-bit client"
+            );
+            msg_printf(NULL, MSG_INFO,
+                "  (to use VirtualBox, install a 64-bit BOINC client)."
+            );
+        }
+#endif
     }
 }
 
 int rsc_index(const char* name) {
     const char* nm = strcmp(name, "CUDA")?name:GPU_TYPE_NVIDIA;
+        // handle old state files
     for (int i=0; i<coprocs.n_rsc; i++) {
         if (!strcmp(nm, coprocs.coprocs[i].type)) {
             return i;
@@ -208,22 +268,18 @@ int rsc_index(const char* name) {
     return -1;
 }
 
+// used in XML and COPROC::type
+//
 const char* rsc_name(int i) {
     return coprocs.coprocs[i].type;
 }
 
-// set no_X_apps for anonymous platform project
+// user-friendly version
 //
-static void check_no_apps(PROJECT* p) {
-    for (int i=0; i<coprocs.n_rsc; i++) {
-        p->no_rsc_apps[i] = true;
-    }
-
-    for (unsigned int i=0; i<gstate.app_versions.size(); i++) {
-        APP_VERSION* avp = gstate.app_versions[i];
-        if (avp->project != p) continue;
-        p->no_rsc_apps[avp->gpu_usage.rsc_type] = false;
-    }
+const char* rsc_name_long(int i) {
+    int num = coproc_type_name_to_num(coprocs.coprocs[i].type);
+    if (num >= 0) return proc_type_name(num);   // CPU, NVIDIA GPU, AMD GPU or Intel GPU
+    return coprocs.coprocs[i].type;             // Some other type
 }
 
 // alert user if any jobs need more RAM than available
@@ -267,7 +323,13 @@ void CLIENT_STATE::set_now() {
 
     // if time went backward significantly, clear delays
     //
+    clock_change = false;
     if (x < (now-60)) {
+        clock_change = true;
+        msg_printf(NULL, MSG_INFO,
+            "New system time (%.0f) < old system time (%.0f); clearing timeouts",
+            x, now
+        );
         clear_absolute_times();
     }
 
@@ -296,11 +358,14 @@ int CLIENT_STATE::init() {
 
     srand((unsigned int)time(0));
     now = dtime();
-    client_start_time = now;
+#ifdef ANDROID
+    device_status_time = dtime();
+#endif
     scheduler_op->url_random = drand();
 
     notices.init();
     daily_xfer_history.init();
+    time_stats.init();
 
     detect_platforms();
     time_stats.start();
@@ -343,13 +408,17 @@ int CLIENT_STATE::init() {
     msg_printf(NULL, MSG_INFO, "Running under account %s", pbuf);
 #endif
 
+    FILE* f = fopen(CLIENT_BRAND_FILENAME, "r");
+    if (f) {
+        fgets(client_brand, sizeof(client_brand), f);
+        strip_whitespace(client_brand);
+        msg_printf(NULL, MSG_INFO, "Client brand: %s", client_brand);
+        fclose(f);
+    }
+
     parse_account_files();
     parse_statistics_files();
 
-    host_info.get_host_info();
-    set_ncpus();
-    show_host_info();
-
     // check for GPUs.
     //
     for (int j=1; j<coprocs.n_rsc; j++) {
@@ -358,7 +427,7 @@ int CLIENT_STATE::init() {
             coprocs.coprocs[j].type
         );
     }
-    if (!config.no_gpus
+    if (!cc_config.no_gpus
 #ifdef _WIN32
         && !executing_as_daemon
 #endif
@@ -366,8 +435,7 @@ int CLIENT_STATE::init() {
         vector<string> descs;
         vector<string> warnings;
         coprocs.get(
-            config.use_all_gpus, descs, warnings,
-            config.ignore_nvidia_dev, config.ignore_ati_dev
+            cc_config.use_all_gpus, descs, warnings, cc_config.ignore_gpu_instance
         );
         for (i=0; i<descs.size(); i++) {
             msg_printf(NULL, MSG_INFO, "%s", descs[i].c_str());
@@ -379,11 +447,15 @@ int CLIENT_STATE::init() {
         }
 #if 0
         msg_printf(NULL, MSG_INFO, "Faking an NVIDIA GPU");
-        coprocs.nvidia.fake(18000, 512*MEGA, 490*MEGA, 1);
+        coprocs.nvidia.fake(18000, 512*MEGA, 490*MEGA, 2);
 #endif
 #if 0
         msg_printf(NULL, MSG_INFO, "Faking an ATI GPU");
         coprocs.ati.fake(512*MEGA, 256*MEGA, 2);
+#endif
+#if 0
+        msg_printf(NULL, MSG_INFO, "Faking an Intel GPU");
+        coprocs.intel_gpu.fake(512*MEGA, 256*MEGA, 2);
 #endif
     }
 
@@ -401,7 +473,16 @@ int CLIENT_STATE::init() {
             coprocs.add(coprocs.ati);
         }
     }
-    host_info._coprocs = coprocs;
+    if (coprocs.have_intel_gpu()) {
+        if (rsc_index(GPU_TYPE_INTEL)>0) {
+            msg_printf(NULL, MSG_INFO, "INTEL GPU info taken from cc_config.xml");
+        } else {
+            coprocs.add(coprocs.intel_gpu);
+        }
+    }
+    coprocs.add_other_coproc_types();
+    
+    host_info.coprocs = coprocs;
     
     if (coprocs.none() ) {
         msg_printf(NULL, MSG_INFO, "No usable GPUs found");
@@ -412,7 +493,7 @@ int CLIENT_STATE::init() {
     // check for app_info.xml file in project dirs.
     // If find, read app info from there, set project.anonymous_platform
     // - this must follow coproc.get() (need to know if GPUs are present)
-    // - this is being done before CPU speed has been read,
+    // - this is being done before CPU speed has been read from state file,
     // so we'll need to patch up avp->flops later;
     //
     check_anonymous();
@@ -427,10 +508,21 @@ int CLIENT_STATE::init() {
     //
     parse_state_file();
 
-    // this needs to go after parse_state_file because
+    // this follows parse_state_file() since we need to have read
+    // domain_name for Android
+    //
+    host_info.get_host_info();
+    set_ncpus();
+    show_host_info();
+
+    // check for app_config.xml files in project dirs
+    //
+    check_app_config();
+
+    // this needs to go after parse_state_file() because
     // GPU exclusions refer to projects
     //
-    config.show();
+    cc_config.show();
 
     // inform the user if there's a newer version of client
     //
@@ -441,12 +533,15 @@ int CLIENT_STATE::init() {
     //
     parse_account_files_venue();
 
-    // fill in p->no_X_apps for anon platform projects
+    // fill in p->no_X_apps for anon platform projects,
+    // and check no_rsc_apps for others
     //
     for (i=0; i<projects.size(); i++) {
         p = projects[i];
         if (p->anonymous_platform) {
-            check_no_apps(p);
+            p->check_no_apps();
+        } else {
+            p->check_no_rsc_apps();
         }
     }
 
@@ -461,10 +556,10 @@ int CLIENT_STATE::init() {
             avp->flops = avp->avg_ncpus * host_info.p_fpops;
 
             // for GPU apps, use conservative estimate:
-            // assume app will run at peak CPU speed, not peak GPU
+            // assume GPU runs at 10X peak CPU speed
             //
             if (avp->gpu_usage.rsc_type) {
-                avp->flops += avp->gpu_usage.usage * host_info.p_fpops;
+                avp->flops += avp->gpu_usage.usage * 10 * host_info.p_fpops;
             }
         }
     }
@@ -520,11 +615,13 @@ int CLIENT_STATE::init() {
     }
     // if new version of client,
     // - run CPU benchmarks
+    // - get new project list
     // - contact reference site (or some project) to trigger firewall alert
     //
     if (new_client) {
         run_cpu_benchmarks = true;
-        if (config.dont_contact_ref_site) {
+        all_projects_list_check_time = 0;
+        if (cc_config.dont_contact_ref_site) {
             if (projects.size() > 0) {
                 projects[0]->master_url_fetch_pending = true;
             }
@@ -533,6 +630,8 @@ int CLIENT_STATE::init() {
         }
     }
 
+    check_if_need_benchmarks();
+
     log_show_projects();
 
     read_global_prefs();
@@ -542,7 +641,7 @@ int CLIENT_STATE::init() {
     request_schedule_cpus("Startup");
     request_work_fetch("Startup");
     work_fetch.init();
-    debt_interval_start = now;
+    rec_interval_start = now;
 
     // set up the project and slot directories
     //
@@ -559,29 +658,32 @@ int CLIENT_STATE::init() {
     //
     set_client_state_dirty("init");
 
-    // initialize GUI RPC data structures before we start accepting
-    // GUI RPC's.
+    // check for initialization files
     //
     acct_mgr_info.init();
     project_init.init();
 
+    // set up for handling GUI RPCs
+    //
     if (!no_gui_rpc) {
-        // When we're running at boot time,
-        // it may be a few seconds before we can socket/bind/listen.
-        // So retry a few times.
-        //
-        for (i=0; i<30; i++) {
-            bool last_time = (i==29);
-            retval = gui_rpcs.init(last_time);
-            if (!retval) break;
-            boinc_sleep(1.0);
+        if (gui_rpc_unix_domain) {
+            retval = gui_rpcs.init_unix_domain();
+        } else {
+            // When we're running at boot time,
+            // it may be a few seconds before we can socket/bind/listen.
+            // So retry a few times.
+            //
+            for (i=0; i<30; i++) {
+                bool last_time = (i==29);
+                retval = gui_rpcs.init_tcp(last_time);
+                if (!retval) break;
+                boinc_sleep(1.0);
+            }
         }
         if (retval) return retval;
     }
 
-#ifdef SANDBOX
-    get_project_gid();
-#endif
+    if (g_use_sandbox) get_project_gid();
 #ifdef _WIN32
     get_sandbox_account_service_token();
     if (sandbox_account_service_token != NULL) g_use_sandbox = true;
@@ -597,11 +699,29 @@ int CLIENT_STATE::init() {
 #endif
 
     http_ops->cleanup_temp_files();
+    
+    // must parse env vars after parsing state file
+    // otherwise items will get overwritten with state file info
+    //
+    parse_env_vars();
+
+    // do this after parsing env vars
+    //
+    proxy_info_startup();
+
+    if (gstate.projects.size() == 0) {
+        msg_printf(NULL, MSG_INFO,
+            "This computer is not attached to any projects"
+        );
+        msg_printf(NULL, MSG_INFO,
+            "Visit http://boinc.berkeley.edu for instructions"
+        );
+    }
 
     // get list of BOINC projects occasionally,
     // and initialize notice RSS feeds
     //
-    if (!config.no_info_fetch) {
+    if (!cc_config.no_info_fetch) {
         all_projects_list_check();
         notices.init_rss();
     }
@@ -614,6 +734,10 @@ int CLIENT_STATE::init() {
     //
     project_priority_init(false);
 
+#ifdef NEW_CPU_THROTTLE
+    client_mutex.lock();
+    throttle_thread.run(throttler, NULL);
+#endif
     initialized = true;
     return 0;
 }
@@ -645,12 +769,18 @@ void CLIENT_STATE::do_io_or_sleep(double x) {
         all_fds = curl_fds;
         gui_rpcs.get_fdset(gui_rpc_fds, all_fds);
         double_to_timeval(action?0:x, tv);
+#ifdef NEW_CPU_THROTTLE
+        client_mutex.unlock();
+#endif
         n = select(
             all_fds.max_fd+1,
             &all_fds.read_fds, &all_fds.write_fds, &all_fds.exc_fds,
             &tv
         );
         //printf("select in %d out %d\n", all_fds.max_fd, n);
+#ifdef NEW_CPU_THROTTLE
+        client_mutex.lock();
+#endif
 
         // Note: curl apparently likes to have curl_multi_perform()
         // (called from net_xfers->got_select())
@@ -721,18 +851,29 @@ bool CLIENT_STATE::poll_slow_events() {
         last_wakeup_time = now;
     }
 
-    if (should_run_cpu_benchmarks() && !are_cpu_benchmarks_running()) {
+    if (run_cpu_benchmarks && can_run_cpu_benchmarks()) {
         run_cpu_benchmarks = false;
         start_cpu_benchmarks();
     }
 
+#ifdef _WIN32
+    if (have_sysmon_msg) {
+        msg_printf(NULL, MSG_INFO, sysmon_msg);
+        have_sysmon_msg = false;
+    }
+#endif
+
     bool old_user_active = user_active;
+#ifdef ANDROID
+    user_active = device_status.user_active;
+#else
     user_active = !host_info.users_idle(
         check_all_logins, global_prefs.idle_time_to_run
 #ifdef __APPLE__
          , &idletime
 #endif
     );
+#endif
 
     if (user_active != old_user_active) {
         request_schedule_cpus("Idle state change");
@@ -781,11 +922,12 @@ bool CLIENT_STATE::poll_slow_events() {
     if (tasks_restarted) {
         if (suspend_reason) {
             if (!tasks_suspended) {
-                suspend_tasks(suspend_reason);
+                show_suspend_tasks_message(suspend_reason);
+                active_tasks.suspend_all(suspend_reason);
             }
             last_suspend_reason = suspend_reason;
         } else {
-            if (tasks_suspended) {
+            if (tasks_suspended && !tasks_throttled) {
                 resume_tasks(last_suspend_reason);
             }
         }
@@ -794,12 +936,12 @@ bool CLIENT_STATE::poll_slow_events() {
         //
         first = false;
         if (suspend_reason) {
-            print_suspend_tasks_message(suspend_reason);
+            show_suspend_tasks_message(suspend_reason);
         }
     }
     tasks_suspended = (suspend_reason != 0);
 
-    if (suspend_reason & SUSPEND_REASON_BENCHMARKS) {
+    if (benchmarks_running) {
         cpu_benchmarks_poll();
     }
 
@@ -883,7 +1025,7 @@ bool CLIENT_STATE::poll_slow_events() {
             //      handle transient and permanent failures
             //      delete the FILE_XFER
 
-        if (!config.no_info_fetch) {
+        if (!cc_config.no_info_fetch) {
             POLL_ACTION(rss_feed_op            , rss_feed_op.poll );
         }
     }
@@ -1057,7 +1199,7 @@ int CLIENT_STATE::link_app_version(PROJECT* p, APP_VERSION* avp) {
         }
 
         if (!strcmp(file_ref.open_name, GRAPHICS_APP_FILENAME)) {
-            char relpath[512], path[512];
+            char relpath[MAXPATHLEN], path[MAXPATHLEN];
             get_pathname(fip, relpath, sizeof(relpath));
             relative_to_absolute(relpath, path);
             strlcpy(avp->graphics_exec_path, path, sizeof(avp->graphics_exec_path));
@@ -1066,7 +1208,7 @@ int CLIENT_STATE::link_app_version(PROJECT* p, APP_VERSION* avp) {
 
         // any file associated with an app version must be signed
         //
-        if (!config.unsigned_apps_ok) {
+        if (!cc_config.unsigned_apps_ok) {
             fip->signature_required = true;
         }
 
@@ -1203,7 +1345,7 @@ bool CLIENT_STATE::abort_unstarted_late_jobs() {
             "Aborting task %s; not started and deadline has passed",
             rp->name
         );
-        rp->abort_inactive(ERR_UNSTARTED_LATE);
+        rp->abort_inactive(EXIT_UNSTARTED_LATE);
         action = true;
     }
     return action;
@@ -1212,7 +1354,7 @@ bool CLIENT_STATE::abort_unstarted_late_jobs() {
 bool CLIENT_STATE::garbage_collect() {
     bool action;
     static double last_time=0;
-    if (gstate.now - last_time < GARBAGE_COLLECT_PERIOD) return false;
+    if (!clock_change && now - last_time < GARBAGE_COLLECT_PERIOD) return false;
     last_time = gstate.now;
 
     action = abort_unstarted_late_jobs();
@@ -1324,6 +1466,7 @@ bool CLIENT_STATE::garbage_collect_always() {
                         rp->name
                     );
                 }
+                add_old_result(*rp);
                 delete rp;
                 result_iter = results.erase(result_iter);
                 action = true;
@@ -1413,7 +1556,8 @@ bool CLIENT_STATE::garbage_collect_always() {
 
     // go through APP_VERSIONs;
     // delete any not referenced by any WORKUNIT
-    // and superceded by a more recent version.
+    // and superceded by a more recent version
+    // for the same platform and plan class
     //
     avp_iter = app_versions.begin();
     while (avp_iter != app_versions.end()) {
@@ -1422,9 +1566,10 @@ bool CLIENT_STATE::garbage_collect_always() {
             found = false;
             for (j=0; j<app_versions.size(); j++) {
                 avp2 = app_versions[j];
-                if (avp2->app==avp->app
+                if (avp2->app == avp->app
+                    && avp2->version_num > avp->version_num
                     && (!strcmp(avp2->plan_class, avp->plan_class))
-                    && avp2->version_num>avp->version_num
+                    && (!strcmp(avp2->platform, avp->platform))
                 ) {
                     found = true;
                     break;
@@ -1518,8 +1663,8 @@ bool CLIENT_STATE::update_results() {
     static double last_time=0;
     int retval;
 
-    if (gstate.now - last_time < UPDATE_RESULTS_PERIOD) return false;
-    last_time = gstate.now;
+    if (!clock_change && now - last_time < UPDATE_RESULTS_PERIOD) return false;
+    last_time = now;
 
     result_iter = results.begin();
     while (result_iter != results.end()) {
@@ -1553,7 +1698,6 @@ bool CLIENT_STATE::update_results() {
             if (rp->is_upload_done()) {
                 rp->set_ready_to_report();
                 rp->completed_time = gstate.now;
-                rp->project->last_upload_start = 0;
                 rp->set_state(RESULT_FILES_UPLOADED, "CS::update_results");
 
                 // clear backoffs for app's resources;
@@ -1593,7 +1737,7 @@ bool CLIENT_STATE::time_to_exit() {
         );
         return true;
     }
-    if (config.exit_when_idle
+    if (cc_config.exit_when_idle
         && (results.size() == 0)
         && had_or_requested_work
     ) {
@@ -1647,9 +1791,9 @@ int CLIENT_STATE::report_result_error(RESULT& res, const char* format, ...) {
     vsnprintf(err_msg, sizeof(err_msg), format, va);
     va_end(va);
 
-    sprintf(buf, "Unrecoverable error for task %s (%s)", res.name, err_msg);
+    sprintf(buf, "Unrecoverable error for task %s", res.name);
 #ifndef SIM
-    scheduler_op->backoff(res.project, buf);
+    scheduler_op->project_rpc_backoff(res.project, buf);
 #endif
 
     sprintf( buf, "<message>\n%s\n</message>\n", err_msg);
@@ -1720,12 +1864,11 @@ int CLIENT_STATE::report_result_error(RESULT& res, const char* format, ...) {
 // - stop all active tasks
 // - stop all file transfers
 // - stop scheduler RPC if any
-// - delete all workunits and results
-// - delete all apps and app_versions
+// - delete workunits and results
+// - delete apps and app_versions
 // - garbage collect to delete unneeded files
-// - clear debts and backoffs
+// - clear backoffs
 //
-// Note: does NOT delete persistent files or user-supplied files;
 // does not delete project dir
 //
 int CLIENT_STATE::reset_project(PROJECT* project, bool detaching) {
@@ -1754,7 +1897,6 @@ int CLIENT_STATE::reset_project(PROJECT* project, bool detaching) {
             i--;
         }
     }
-    project->last_upload_start = 0;
 
     // if we're in the middle of a scheduler op to the project, abort it
     //
@@ -1778,6 +1920,15 @@ int CLIENT_STATE::reset_project(PROJECT* project, bool detaching) {
     project->user_files.clear();
     project->project_files.clear();
 
+    // clear flags so that sticky files get deleted
+    //
+    for (i=0; i<file_infos.size(); i++) {
+        FILE_INFO* fip = file_infos[i];
+        if (fip->project == project) {
+            fip->sticky = false;
+        }
+    }
+
     garbage_collect_always();
 
     // remove apps and app_versions (but not if anonymous platform)
@@ -1807,6 +1958,10 @@ int CLIENT_STATE::reset_project(PROJECT* project, bool detaching) {
         garbage_collect_always();
     }
 
+    // force refresh of scheduler URLs
+    //
+    project->scheduler_urls.clear();
+
     project->duration_correction_factor = 1;
     project->ams_resource_share = -1;
     project->min_rpc_time = 0;
@@ -1823,13 +1978,14 @@ int CLIENT_STATE::reset_project(PROJECT* project, bool detaching) {
 // - delete all file infos
 // - delete account file
 // - delete project directory
+// - delete various per-project files
 //
 int CLIENT_STATE::detach_project(PROJECT* project) {
     vector<PROJECT*>::iterator project_iter;
     vector<FILE_INFO*>::iterator fi_iter;
     FILE_INFO* fip;
     PROJECT* p;
-    char path[256];
+    char path[MAXPATHLEN];
     int retval;
 
     reset_project(project, true);
@@ -1849,14 +2005,6 @@ int CLIENT_STATE::detach_project(PROJECT* project) {
         }
     }
 
-    // if global prefs came from this project, delete file and reinit
-    //
-    p = lookup_project(global_prefs.source_project);
-    if (p == project) {
-        boinc_delete_file(GLOBAL_PREFS_FILE_NAME);
-        global_prefs.defaults();
-    }
-
     // find project and remove it from the vector
     //
     for (project_iter = projects.begin(); project_iter != projects.end(); project_iter++) {
@@ -1905,6 +2053,12 @@ int CLIENT_STATE::detach_project(PROJECT* project) {
         );
     }
 
+    // remove miscellaneous per-project files
+    //
+    //job_log_filename(*project, path, sizeof(path));
+    //boinc_delete_file(path);
+    delete_project_notice_files(project);
+
     rss_feeds.update_feed_list();
 
     delete project;
@@ -1922,7 +2076,7 @@ int CLIENT_STATE::detach_project(PROJECT* project) {
 // e.g. flush buffers, but why bother)
 //
 int CLIENT_STATE::quit_activities() {
-    // calculate long-term debts (for state file)
+    // calculate REC (for state file)
     //
     adjust_rec();
 
@@ -1941,13 +2095,17 @@ int CLIENT_STATE::quit_activities() {
 
 #endif
 
-// See if a timestamp in the client state file
+// Called at startup to see if a timestamp in the client state file
 // is later than the current time.
 // If so, the user must have decremented the system clock.
 //
 void CLIENT_STATE::check_clock_reset() {
     if (!time_stats.last_update) return;
     if (time_stats.last_update <= now) return;
+    msg_printf(NULL, MSG_INFO,
+        "System clock (%.0f) < state file timestamp (%.0f); clearing timeouts",
+        now, time_stats.last_update
+    );
     clear_absolute_times();
 }
 
@@ -1959,10 +2117,8 @@ void CLIENT_STATE::check_clock_reset() {
 // that we could try to patch up, but it's not clear how.
 //
 void CLIENT_STATE::clear_absolute_times() {
-    msg_printf(NULL, MSG_INFO,
-        "System clock was turned backwards; clearing timeouts"
-    );
-
+    exclusive_app_running = 0;
+    exclusive_gpu_app_running = 0;
     new_version_check_time = now;
     all_projects_list_check_time = now;
     retry_shmem_time = 0;
@@ -2014,6 +2170,7 @@ void CLIENT_STATE::log_show_projects() {
         if (p->ended) {
             msg_printf(p, MSG_INFO, "Project has ended - OK to detach");
         }
+        p->show_no_work_notice();
     }
 }
 
@@ -2034,9 +2191,9 @@ void CLIENT_STATE::start_abort_sequence() {
         if (rp->computing_done()) continue;
         ACTIVE_TASK* atp = lookup_active_task_by_result(rp);
         if (atp) {
-            atp->abort_task(ERR_ABORTED_ON_EXIT, "aborting on client exit");
+            atp->abort_task(EXIT_CLIENT_EXITING, "aborting on client exit");
         } else {
-            rp->abort_inactive(ERR_ABORTED_ON_EXIT);
+            rp->abort_inactive(EXIT_CLIENT_EXITING);
         }
     }
     for (i=0; i<projects.size(); i++) {
