@@ -27,8 +27,9 @@
 #ifndef _WIN32
 #include "config.h"
 #include <string>
-#include <math.h>
+#include <cmath>
 #include <string.h>
+#include <time.h>
 #include <stdlib.h>
 #include <ctype.h>
 #endif
@@ -80,7 +81,7 @@ size_t strlcat(char *dst, const char *src, size_t size) {
 
 #if !HAVE_STRCASESTR
 // BOINC only uses strcasestr() for short strings,
-// so the following till suffice
+// so the following will suffice
 //
 const char *strcasestr(const char *s1, const char *s2) {
     char needle[1024], haystack[1024], *p=NULL;
@@ -330,9 +331,13 @@ void strip_whitespace(string& str) {
 
 char* time_to_string(double t) {
     static char buf[100];
-    time_t x = (time_t)t;
-    struct tm* tm = localtime(&x);
-    strftime(buf, sizeof(buf)-1, "%d-%b-%Y %H:%M:%S", tm);
+    if (!t) {
+        strcpy(buf, "---");
+    } else {
+        time_t x = (time_t)t;
+        struct tm* tm = localtime(&x);
+        strftime(buf, sizeof(buf)-1, "%d-%b-%Y %H:%M:%S", tm);
+    }
     return buf;
 }
 
@@ -362,33 +367,27 @@ string timediff_format(double diff) {
     int sex = tdiff % 60;
     tdiff /= 60;
     if (!tdiff) {
-        sprintf(buf, "%d sec", sex);
+        sprintf(buf, "00:00:%02d", sex);
         return buf;
     }
 
     int min = tdiff % 60;
     tdiff /= 60;
     if (!tdiff) {
-        sprintf(buf, "%d min %d sec", min, sex);
+        sprintf(buf, "00:%02d:%02d", min, sex);
         return buf;
     }
 
     int hours = tdiff % 24;
     tdiff /= 24;
     if (!tdiff) {
-        sprintf(buf, "%d hr %d min %d sec", hours, min, sex);
+        sprintf(buf, "%02d:%02d:%02d", hours, min, sex);
         return buf;
     }
 
-    int days = tdiff % 7;
-    tdiff /= 7;
-    if (!tdiff) {
-        sprintf(buf, "%d days %d hr %d min %d sec", days, hours, min, sex);
-        return buf;
-    }
-
-    sprintf(buf, "%d weeks %d days %d hrs %d min %d sec", (int)tdiff, days, hours, min, sex);
+    sprintf(buf, "%d days %02d:%02d:%02d", tdiff, hours, min, sex);
     return buf;
+
 }
 
 void mysql_timestamp(double dt, char* p) {
@@ -433,6 +432,7 @@ const char* boincerror(int which_error) {
         case ERR_NO_SIGNATURE: return "no signature";
         case ERR_THREAD: return "thread failure";
         case ERR_SIGNAL_CATCH: return "caught signal";
+        case ERR_BAD_FORMAT: return "bad file format";
         case ERR_UPLOAD_TRANSIENT: return "transient upload error";
         case ERR_UPLOAD_PERMANENT: return "permanent upload error";
         case ERR_IDLE_PERIOD: return "user preferences say can't start work";
@@ -527,6 +527,7 @@ const char* boincerror(int which_error) {
         case ERR_CRYPTO: return "encryption error";
         case ERR_ABORTED_ON_EXIT: return "job was aborted on client exit";
         case ERR_PROC_PARSE: return "a /proc entry was not parsed correctly";
+        case ERR_PIPE: return "pipe() failed";
         case 404: return "HTTP file not found";
         case 407: return "HTTP proxy authentication failure";
         case 416: return "HTTP range request error";
@@ -542,13 +543,13 @@ const char* boincerror(int which_error) {
 }
 
 const char* network_status_string(int n) {
-	switch (n) {
-	case NETWORK_STATUS_ONLINE: return "online";
-	case NETWORK_STATUS_WANT_CONNECTION: return "need connection";
-	case NETWORK_STATUS_WANT_DISCONNECT: return "don't need connection";
-	case NETWORK_STATUS_LOOKUP_PENDING: return "reference site lookup pending";
-	default: return "unknown";
-	}
+    switch (n) {
+    case NETWORK_STATUS_ONLINE: return "online";
+    case NETWORK_STATUS_WANT_CONNECTION: return "need connection";
+    case NETWORK_STATUS_WANT_DISCONNECT: return "don't need connection";
+    case NETWORK_STATUS_LOOKUP_PENDING: return "reference site lookup pending";
+    default: return "unknown";
+    }
 }
 
 const char* rpc_reason_string(int reason) {
@@ -576,11 +577,12 @@ const char* suspend_reason_string(int reason) {
     case SUSPEND_REASON_INITIAL_DELAY: return "initial delay";
     case SUSPEND_REASON_EXCLUSIVE_APP_RUNNING: return "an exclusive app is running";
     case SUSPEND_REASON_CPU_USAGE: return "CPU is busy";
-    case SUSPEND_REASON_NETWORK_QUOTA_EXCEEDED: return "network bandwidth limit exceeded";
+    case SUSPEND_REASON_NETWORK_QUOTA_EXCEEDED: return "network transfer limit exceeded";
     case SUSPEND_REASON_OS: return "requested by operating system";
     case SUSPEND_REASON_WIFI_STATE: return "not connected to WiFi network";
-    case SUSPEND_REASON_BATTERY_CHARGING: return "battery is recharging";
-    case SUSPEND_REASON_BATTERY_OVERHEATED: return "battery is overheated";
+    case SUSPEND_REASON_BATTERY_CHARGING: return "battery low";
+    case SUSPEND_REASON_BATTERY_OVERHEATED: return "battery thermal protection";
+    case SUSPEND_REASON_NO_GUI_KEEPALIVE: return "GUI not active";
     }
     return "unknown reason";
 }
@@ -594,6 +596,67 @@ const char* run_mode_string(int mode) {
     return "unknown";
 }
 
+const char* battery_state_string(int state) {
+    switch (state) {
+    case BATTERY_STATE_DISCHARGING: return "discharging";
+    case BATTERY_STATE_CHARGING: return "charging";
+    case BATTERY_STATE_FULL: return "full";
+    case BATTERY_STATE_OVERHEATED: return "overheated";
+    }
+    return "unknown";
+}
+
+const char* result_client_state_string(int state) {
+    switch (state) {
+    case RESULT_NEW: return "new";
+    case RESULT_FILES_DOWNLOADING: return "downloading";
+    case RESULT_FILES_DOWNLOADED: return "downloaded";
+    case RESULT_COMPUTE_ERROR: return "compute error";
+    case RESULT_FILES_UPLOADING: return "uploading";
+    case RESULT_FILES_UPLOADED: return "uploaded";
+    case RESULT_ABORTED: return "aborted";
+    case RESULT_UPLOAD_FAILED: return "upload failed";
+    }
+    return "unknown";
+}
+
+const char* result_scheduler_state_string(int state) {
+    switch (state) {
+    case CPU_SCHED_UNINITIALIZED: return "uninitialized";
+    case CPU_SCHED_PREEMPTED: return "preempted";
+    case CPU_SCHED_SCHEDULED: return "scheduled";
+    }
+    return "unknown";
+}
+
+const char* active_task_state_string(int state) {
+    switch (state) {
+    case PROCESS_UNINITIALIZED: return "UNINITIALIZED";
+    case PROCESS_EXECUTING: return "EXECUTING";
+    case PROCESS_SUSPENDED: return "SUSPENDED";
+    case PROCESS_ABORT_PENDING: return "ABORT_PENDING";
+    case PROCESS_EXITED: return "EXITED";
+    case PROCESS_WAS_SIGNALED: return "WAS_SIGNALED";
+    case PROCESS_EXIT_UNKNOWN: return "EXIT_UNKNOWN";
+    case PROCESS_ABORTED: return "ABORTED";
+    case PROCESS_COULDNT_START: return "COULDNT_START";
+    case PROCESS_QUIT_PENDING: return "QUIT_PENDING";
+    case PROCESS_COPY_PENDING: return "COPY_PENDING";
+    }
+    return "Unknown";
+}
+
+const char* batch_state_string(int state) {
+    switch (state) {
+    case BATCH_STATE_INIT: return "uninitialized";
+    case BATCH_STATE_IN_PROGRESS: return "in progress";
+    case BATCH_STATE_COMPLETE: return "completed";
+    case BATCH_STATE_ABORTED: return "aborted";
+    case BATCH_STATE_RETIRED: return "retired";
+    }
+    return "unknown";
+}
+
 // string substitution:
 // haystack is the input string
 // out is the output buffer
@@ -636,9 +699,20 @@ inline void remove_str(char* p, const char* str) {
     }
 }
 
-// remove _( and ") from string
+// remove _(" and ") from string
 //
 void strip_translation(char* p) {
     remove_str(p, "_(\"");
     remove_str(p, "\")");
 }
+
+char* lf_terminate(char* p) {
+    int n = (int)strlen(p);
+    if (p[n-1] == '\n') {
+        return p;
+    }
+    p = (char*)realloc(p, n+2);
+    p[n] = '\n';
+    p[n+1] = 0;
+    return p;
+}
